package au.com.infomedix.documentstore.cnd;

import org.modeshape.jcr.JcrConfiguration;
import org.modeshape.jcr.JcrEngine;
import org.modeshape.jcr.api.nodetype.NodeTypeManager;
import java.io.InputStream;
import java.io.PrintStream;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Workspace;
import javax.jcr.version.VersionManager;


public class TestSharedVersioned
{
    private static Session session;

    public static void main(String[] args)
    {
        JcrEngine engine = null;

        try
        {
            InputStream      myConfigStream = TestLoadCND.class.getClassLoader().getResourceAsStream("modeshape-config.xml");
            JcrConfiguration config         = new JcrConfiguration().loadFrom(myConfigStream);
            engine = config.build();
            engine.start();

            Repository repository = engine.getRepository("CPF repository");
            session = repository.login();

            Workspace      workspace      = session.getWorkspace();
            VersionManager versionManager = workspace.getVersionManager();

            NodeTypeManager nodeTypeManager = (NodeTypeManager) workspace.getNodeTypeManager();
            System.out.println("Parse the cnd file");

            InputStream myCndStream = TestLoadCND.class.getClassLoader().getResourceAsStream("simple-shared.cnd");
            nodeTypeManager.registerNodeTypes(myCndStream, false);

            Node root = session.getRootNode();
            Node a    = root.addNode("a", "A");
            a.setProperty("name", "aaa");
            session.save();
            versionManager.checkin(a.getPath());

            listNodes("/a", "1) ", "jcr:uuid", System.out);

            versionManager.checkout(a.getPath());

            Node b = a.addNode("b", "B");
            b.setProperty("name", "bbb");
            session.save();
            versionManager.checkin(a.getPath());

            listNodes("/a", "2) ", "jcr:uuid", System.out);

            versionManager.checkout(a.getPath());

            Node c = b.addNode("c", "C");
            c.setProperty("name", "ccc");
            session.save();
            versionManager.checkin(a.getPath());

            listNodes("/a", "3) ", "jcr:uuid", System.out);

            versionManager.checkout(a.getPath());
            workspace.clone(workspace.getName(), c.getPath(), "/a/c", false);
            listNodes("/", "4) ", "jcr:uuid", System.out);
            versionManager.checkin(a.getPath());

            listNodes("/", "5) ", "jcr:uuid", System.out);
        }
        catch(Exception e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        finally
        {
            if(engine != null)
            {
                engine.shutdown();
            }
        }
    }

    /**
     * Recursively list all child nodes.
     * @throws RepositoryException
     */
    public static void listNodes(String      npath,
                                 String      indent,
                                 String      uuidName,
                                 PrintStream out)
                          throws RepositoryException
    {
        Node n = session.getNode(npath);
        if(n.isNodeType("mode:system"))
        {
            return;
        }
        out.println(indent + (n.hasProperty("name") ? n.getProperty("name") : "(noname)") + " "
                    + (n.hasProperty(uuidName) ? n.getProperty(uuidName) : "(nouuid)") + " " + n);

        NodeIterator iter = n.getNodes();

        while(iter.hasNext())
        {
            Node node = iter.nextNode();
            listNodes(node.getPath(), indent + "  ", uuidName, out);
        }
    }
}